
local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local plugin = RW:Plugin("Bite Buddy")

----------------------------
-- Locals

local LB = LibStub("AceLocale-3.0"):GetLocale("RW2-Plugin-Base")
local LSM = LibStub("LibSharedMedia-3.0")
local db

local BittenSpell = GetSpellInfo(70867)
local DebugSpell = "Horn of Winter"
local Party
local allGroups = {}
local groupsByID = {}
local init = false
local taggedTarget
local pName = UnitName("player")
local Utils = RW.Utils

local insets = 8
local mainBackdrop = {
	tile = false,
	insets = { left = inset, right = inset, top = inset, bottom = inset },
}
local groupBackdrop = {
	tile = false,
	insets = { left = inset, right = inset, top = inset, bottom = inset },
}

local defaults = {
	profile = {
		PluginEnabled = true,
		x = 0,
		y = 0,
		Announce = false,
		Width = 200,
		GroupWidth = 70,
		GroupHeight = 38,
		Horizontal = true,
		BGTexture = "Solid",
		Border = "IshBorder",
		BorderSize = 12,
		Font = "accid",
		FontSize = 12,
		FontFlags = "OUTLINE",
		OutOfRangeOpacity = 0.5,
		FadeOutOfRange = true,
		GroupBGColor = {0.21, 0.41, 0.6, 0.84},
		GroupBGColorHover = {0.28, 0.53, 0.79, 1},
		GroupBorderColor = {1, 1, 1, 1},
		GroupBorderColorHover = {0.45, 0.98, 1, 1},
		GroupBorderColorSelected = {0, 1, 0, 1},
		GroupBorderColorTagged = {1, 0.0, 0.0, 1},
	}
}

----------------------------
-- Init

function plugin:OnRegister()
	self.db = RW.db:RegisterNamespace("BiteBuddy", defaults)
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileChanged", "Reset")
	self.db.RegisterCallback(self, "OnProfileCopied", "Reset")
	self.db.RegisterCallback(self, "OnProfileReset", "Reset")
end

function plugin:PluginEnable()
	if not db.PluginEnabled then
		self:Disable()
		return
	end
	
	self:RegisterMessage("RW_ConfigMode")
	self:RegisterMessage("RW_StartBiteBuddy")
	self:RegisterMessage("RW_StopBiteBuddy")
	
	self:SendMessage("RW_RegisterCommListener", self, "BiteBuddy_TargetSelected")
	self:SendMessage("RW_RegisterCommListener", self, "BiteBuddy_TargetDeSelected")
	
	self:CreateAnchor()
	self:Reset()
	
	Party = RW:GetPlugin("Party")
end

function plugin:PluginDisable()
	if self.frame then
		self.frame:Hide()
	end
end

function plugin:Reset()
	db = self.db.profile
	if plugin.anchor then
		local anchor = plugin.anchor
		local scale = anchor:GetEffectiveScale() / UIParent:GetEffectiveScale()
		anchor:ClearAllPoints()
		anchor:SetPoint("CENTER", UIParent, "CENTER", db.x / scale, db.y / scale)		
		anchor:SetWidth(db.Width)
	end
end

local function dragStart(self)
	self:StartMoving()
end
local function dragStop(self)
	self:StopMovingOrSizing()
	local scale = self:GetEffectiveScale() / UIParent:GetEffectiveScale()
	
	local x, y = self:GetCenter()
	x, y = x * scale, y * scale
	
	x = x - GetScreenWidth()/2
	y = y - GetScreenHeight()/2
	
	db.x = x
	db.y = y
	
	plugin:SendMessage("RW_GUIValuesChanged")
end

function plugin:CreateAnchor()
	local anchor = Utils:CreateAnchor("BiteBuddyAnchor", UIParent, "Bite Buddy")	
	anchor:SetScript("OnDragStart", dragStart)
	anchor:SetScript("OnDragStop", dragStop)
	
	self.anchor = anchor
end

local function EnterGroup(self, event)
	self:SetBackdropColor(unpack(db.GroupBGColorHover))
	self:SetBackdropBorderColor(unpack(db.GroupBorderColorHover))
end

local function LeaveGroup(self, event)
	self:SetBackdropColor(unpack(db.GroupBGColor))
	if self.biterName:GetText() == UnitName("player") then
		self:SetBackdropBorderColor(unpack(db.GroupBorderColorSelected))
	elseif self.tagged then
		self:SetBackdropBorderColor(unpack(db.GroupBorderColorTagged))
	else
		self:SetBackdropBorderColor(unpack(db.GroupBorderColor))
	end
end

local function ClickGroup(self, button)
	local name = self.name:GetText()
	if button == "LeftButton" then
		if self.tagged then
			return 
		end
		local hasBuff = UnitAura("player", BittenSpell)
		
		--[===[@debug@
		if not hasBuff then
			hasBuff = UnitAura("player", DebugSpell)
		end
		--@end-debug@]===]
		
		if true and not taggedTarget and name ~= UnitName("player") then
			self:SetBackdropBorderColor(unpack(db.GroupBorderColorSelected))
			plugin:SendMessage("RW_SendComm", "BiteBuddy_TargetSelected", "RAID", nil, name)
			taggedTarget = name
		end
		
	elseif button == "RightButton" then
		if self.biterName:GetText() == UnitName("player") then
			self:SetBackdropBorderColor(unpack(db.GroupBorderColor))
			taggedTarget = nil
			plugin:SendMessage("RW_SendComm", "BiteBuddy_TargetDeSelected", "RAID", nil, name)
			plugin:SendMessage("RW_HideArrow", "BiteBuddyArrow")
		end
	end
end

function plugin:RemoveTag(unit)
	if groupsByID[unit] then
		groupsByID[unit].biterName:SetText("")
		groupsByID[unit].tagged = false
		groupsByID[unit]:SetBackdropBorderColor(unpack(db.GroupBorderColor))
		groupsByID[unit].removeSchedule = nil
		if groupsByID[unit].name:GetText() == taggedTarget then
			self:SendMessage("RW_HideArrow", "BiteBuddyArrow")
			taggedTarget = nil
		end
	end
end

function plugin:BiteBuddy_TargetSelected(from, dist, name)
	local group
	for id, g in pairs(groupsByID) do
		if UnitName(id) == name then
			group = g
		end
	end
	
	if group then
		if db.Announce then
			SendChatMessage("Bite Buddy: "..from .. " has tagged "..name, "RAID")
		end
		
		if group.tagged then
			group.name:GetText()
			plugin:SendMessage("RW_SendComm", "BiteBuddy_TargetDeSelected", "RAID", nil, name)
			return
		end
		
		if from == pName then
			self:SendMessage("RW_ArrowToNameOrUnit", "BiteBuddyArrow", nil, name, nil, 71726)
		end
	
		group.biterName:SetText(from)
		group.tagged = true
		
		group.removeSchedule = plugin:ScheduleTimer("RemoveTag", 65, group:GetAttribute("unit"))
		
		if group.biterName:GetText() == UnitName("player") then
			group:SetBackdropBorderColor(unpack(db.GroupBorderColorSelected))
		else
			group:SetBackdropBorderColor(unpack(db.GroupBorderColorTagged))
		end
	end
end

function plugin:BiteBuddy_TargetDeSelected(from, dist, name)
	local group
	for id, g in pairs(groupsByID) do
		if UnitName(id) == name then
			group = g
		end
	end
	
	if group then
		if db.Announce then
			SendChatMessage("Bite Buddy: "..from .. " will no longer bite "..name, "RAID")
		end
		
		if group.name:GetText() == taggedTarget then
			self:Msg("Your selected target "..taggedTarget.." was dubble tagged, select a new one!")
			self:SendMessage("RW_Warning", "Double Tag, select new target!", [[Interface\Icons\inv_misc_monsterfang_01]], color, [[Sound\Creature\GurtoggBloodboil\BLCKTMPLE_Gurtogg_Special01.wav]], true)
			taggedTarget = nil
			self:SendMessage("RW_HideArrow", "BiteBuddyArrow")
		end
		
		group.biterName:SetText("")
		group.tagged = false
		group:SetBackdropBorderColor(unpack(db.GroupBorderColor))
		
		if group.removeSchedule then
			self:CancelTimer(group.removeSchedule)
			group.removeSchedule = nil
		end
	end
end

function plugin:CreateGroup(uID)

	frame = CreateFrame("Frame", "BiteBuddyGroup"..(#allGroups + 1), self:CreateMainFrame())
	frame.unit = uID
	
	frame:SetScript("OnEnter", EnterGroup)
	frame:SetScript("OnLeave", LeaveGroup)
	frame:SetScript("OnMouseDown", ClickGroup)
	
	frame:EnableMouse(true)
	
	local str = frame:CreateFontString("BiteBuddyGroupString"..(#allGroups + 1), "ARTWORK")
	str:SetFont(LSM:Fetch("font", db.Font), db.FontSize, db.FontFlags)
	str:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, -3)
	str:SetPoint("TOPRIGHT", frame, "TOPRIGHT", 0, -3)
	str:SetHeight(20)
	frame.name = str
	
	str = frame:CreateFontString("BiteBuddyGroupBiterString"..(#allGroups + 1), "ARTWORK")
	str:SetFont(LSM:Fetch("font", db.Font), db.FontSize, db.FontFlags)
	str:SetPoint("BOTTOMLEFT", frame, "BOTTOMLEFT", 0, 3)
	str:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", 0, 3)
	str:SetHeight(20)
	frame.biterName = str
	
	tinsert(allGroups, frame)
	
	frame.biterName:SetText("")
	frame.biterName:SetTextColor(1, 0, 0)
	frame:SetWidth(db.GroupWidth)
	frame:SetHeight(db.GroupHeight)
	frame:Show()	
	
	return frame
end

function plugin:CreateMainFrame()
	if self.frame then return self.frame end
	local frame = CreateFrame("Frame", "BiteBuddyMainFrame", UIParent)
	frame:SetWidth(db.Width)
	frame:SetHeight(20)
	
	frame:ClearAllPoints()
	frame:SetPoint("BOTTOMLEFT", self.anchor, "TOPLEFT")
	
	frame.elapsed = 0
	
	frame:SetScript("OnUpdate", function(self, elapsed)
		self.elapsed = self.elapsed + elapsed
		if self.elapsed > 0.5 then
			for i, group in pairs(allGroups) do
				if UnitName("raid"..i) then
					if UnitIsDeadOrGhost(group.unit) then
						group:Hide()
						group.dead = true
						if group.name:GetText() == taggedTarget then
							plugin:Msg("Your tagged target, " ..taggedTarget .." has died! Select a new one!")
							taggedTarget = nil
						end
					elseif group.dead and not UnitIsDeadOrGhost(group.unit) then 
						group.dead = false
						group:Show()
					end
					if CheckInteractDistance(group.unit, 3) then
						group:SetAlpha(1)
					elseif db.FadeOutOfRange then
						group:SetAlpha(db.OutOfRangeOpacity)
					else
						group:SetAlpha(1)
					end
				end
			end
			self.elapsed = 0
		end
	end)
	
	self.frame = frame
	
	return frame
end

function plugin:ApplySettings()
	local frame = self:CreateMainFrame()
	
	for i, group in pairs(allGroups) do
		groupBackdrop.bgFile = LSM:Fetch("background", db.BGTexture)
		groupBackdrop.edgeFile = LSM:Fetch("border", db.Border)
		groupBackdrop.edgeSize = db.BorderSize
		group:SetBackdrop(groupBackdrop)
		group:SetBackdropColor(unpack(db.GroupBGColor))
		group:SetBackdropBorderColor(unpack(db.GroupBorderColor))
		group.name:SetFont(LSM:Fetch("font", db.Font), db.FontSize, db.FontFlags)
		group.biterName:SetFont(LSM:Fetch("font", db.Font), db.FontSize, db.FontFlags)
	end
end

do
	local col = {}
	function plugin:SetLayout()
		wipe(col)
		col[1], col[2], col[3], col[4], col[5], col[6], col[7], col[8] = 0, 0, 0, 0, 0, 0, 0, 0
		local frame = self:CreateMainFrame()
		for i, group in pairs(allGroups) do
			local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(i)
			group:ClearAllPoints()
			if name and subgroup then
				if db.Horizontal then
					group:SetPoint("BOTTOMLEFT", frame, "BOTTOMLEFT", (subgroup - 1) * db.GroupWidth, col[subgroup] * db.GroupHeight)					
				else
					group:SetPoint("BOTTOMLEFT", frame, "BOTTOMLEFT", col[subgroup] * db.GroupWidth, (subgroup - 1) * db.GroupHeight)
				end
				group:SetWidth(db.GroupWidth)
				group:SetHeight(db.GroupHeight)
				col[subgroup] = col[subgroup] + 1
			end
		end
	end
end

function plugin:UpdateFrames()
	for i, group in pairs(allGroups) do
		local id = "raid"..i
		local name = UnitName(id)
		if not name then
			group.name:SetText("")
			group:Hide()
		else
			local class = select(2, UnitClass(id))
			local c = RAID_CLASS_COLORS[class]
			group.name:SetText(name)
			if c then
				group.name:SetTextColor(c.r, c.g, c.b)
			end
			if not group.disabled and not group.dead then
				group:Show()	
			end
		end
	end
end

function plugin:RAID_ROSTER_UPDATE()
	self:SetLayout()
	self:UpdateFrames()
end

function plugin:COMBAT_LOG_EVENT_UNFILTERED(_, time, event, sGUID, sName, sFlags, dGUID, dName, dFlags, ...)
	if event == "SPELL_AURA_APPLIED" then
		local spellID, spellName, spellSchool = ...
		if spellName == BittenSpell then
			for id, group in pairs(groupsByID) do
				if UnitName(id) == dName then
					group.disabled = true
					group:Hide()
					
					if taggedTarget == dName and sName ~= pName then
						self:Msg("Your tagged target, " ..taggedTarget .." was bitten by " ..sName ..". Select a new one!")
						taggedTarget = nil
					elseif taggedTarget == dName and sName == pName then
						self:SendMessage("RW_HideArrow", "BiteBuddyArrow")
						taggedTarget = nil
					end
				end
			end
		end
	end
end 

function plugin:RW_StartBiteBuddy()
	if not init then
		for i = 1, 40 do
			local group = self:CreateGroup("raid"..i)
			groupsByID["raid"..i] = group
		end
		init = true
	end
	
	self.frame:Show()
	
	self:SetLayout()
	self:ApplySettings()
	self:UpdateFrames()
	
	taggedTarget = false
	
	self:RegisterEvent("RAID_ROSTER_UPDATE")
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
end

function plugin:RW_StopBiteBuddy()
	if self.frame then
		self.frame:Hide()
	end
	for i, group in pairs(allGroups) do
		group.disabled = false
		group.dead = false
	end
end

function plugin:RW_ConfigMode(message, show)
	if show then
		self.anchor:Show()
		for i, group in pairs(allGroups) do
			group:Show()
		end
	else
		self.anchor:Hide()
		for i, group in pairs(allGroups) do
			group:Hide()
		end
		self:UpdateFrames()
	end
end

function plugin:GetOptions()
	return LB.PLUGINS, LB.BOSS_PLUGINS, "Bite Buddy", self.GetGUI, [[Interface\Icons\inv_misc_monsterfang_01]]
end

local function changeCallback()
	plugin:Reset()
end

local function changeCallback()
	plugin:SetLayout()
	plugin:ApplySettings()
	plugin:Reset()
end

function plugin:GetGUI()
	local self = self or plugin
	local UIF = LibStub("AceGUI-Factory-1.0"):GetFactory("RW")
	local AceGUI = LibStub("AceGUI-3.0")
	
	local group, label, slider, check, igroup, igroup2, button, color, select
	
	group = AceGUI:Create("ScrollFrame")
	group:SetLayout("Flow")
	
	label = UIF:MainTitleLabel("Bite Buddy")
	group:AddChild(label)
	
	igroup = UIF:InlineGroup("")
	group:AddChild(igroup)	
	
	label = UIF:NormalText(
[[This plugin is used in the Blood-Queen Lana'thel fight im Icecrown Citadel.
It will show all targets in the raid that are available to be bitten. All the people who are allready vampires, or dead will not show up.
To select someone simply |cffffff66left click|r the frame in question and you will broadcast to everyone else usng the plugin who you have choosen, and it will show up for them as well. To undo the selection you |cffffff66right click|r the frame.

|cffffff66Note:|r You will not be able to select someone  unless you are a vampire yourself allready. You can also not select more then 1 target, so if you want to switch target you first have to undo your current selection.]])
	label:SetImage([[Interface\Icons\inv_misc_monsterfang_01]])
	igroup:AddChild(label)
	
	igroup = UIF:InlineGroup("")
	group:AddChild(igroup)	
	
	check = UIF:CheckBox(LB.ENABLED, db, "PluginEnabled", function() 
		if not db.PluginEnabled then
			self:Disable()
		else
			self:Enable()
		end
	end)
	igroup:AddChild(check)
	
	check = UIF:CheckBox("Announce to raid", db, "Announce")
	check:SetDescription("Toggle if tagging should be announced to the raid. Only one person in the raid should have this option enabled to prevent flooding the raid chat.")
	igroup:AddChild(check)
	
	check = UIF:CheckBox("Fade out of range", db, "FadeOutOfRange")
	check:SetRelativeWidth(0.48)
	igroup:AddChild(check)
	
	slider = UIF:Slider("Faded Opacity", db, "OutOfRangeOpacity", 0, 1, 0.05, changeCallback)
	slider:SetRelativeWidth(0.48)
	igroup:AddChild(slider)
	
	color = UIF:ColorSelect("Background color", db, "GroupBGColor", changeCallback)
	color:SetHasAlpha(true)
	igroup:AddChild(color)
	
	color = UIF:ColorSelect("Background (mouse over)", db, "GroupBGColorHover", changeCallback)
	color:SetHasAlpha(true)
	igroup:AddChild(color)
	
	igroup2 = UIF:InlineGroup2(LB.BORDER_COLOR)
	igroup:AddChild(igroup2)
	
	color = UIF:ColorSelect("Your Selected target", db, "GroupBorderColorSelected", changeCallback)
	igroup2:AddChild(color)
	
	color = UIF:ColorSelect("Tagged targets", db, "GroupBorderColorTagged", changeCallback)
	igroup2:AddChild(color)
	
	color = UIF:ColorSelect("Free targets", db, "GroupBorderColor", changeCallback)
	igroup2:AddChild(color)
	
	color = UIF:ColorSelect("Mouse Over", db, "GroupBorderColorHover", changeCallback)
	igroup2:AddChild(color)
	
	slider = UIF:Slider(LB.FONT_SIZE, db, "FontSize", 5, 40, 1, changeCallback)
	slider:SetRelativeWidth(0.48)
	igroup:AddChild(slider)
	
	igroup2 = UIF:InlineGroup2(LB.SIZE)
	igroup:AddChild(igroup2)	
	
	slider = UIF:Slider(LB.WIDTH, db, "GroupWidth", 20, 300, 1, changeCallback)
	slider:SetRelativeWidth(0.5)
	igroup2:AddChild(slider)
	
	slider = UIF:Slider(LB.HEIGHT, db, "GroupHeight", 20, 300, 1, changeCallback)
	slider:SetRelativeWidth(0.5)
	igroup2:AddChild(slider)
	
	igroup2 = UIF:InlineGroup2("Position")
	igroup:AddChild(igroup2)	
	
	slider = UIF:Slider("X", db, "x", 
		-floor(GetScreenWidth() * UIParent:GetEffectiveScale()), 
		floor(GetScreenWidth() * UIParent:GetEffectiveScale()),
		1, changeCallback)
	slider:SetRelativeWidth(0.5)
	igroup2:AddChild(slider)
	
	slider = UIF:Slider("Y", db, "y", 
		-floor(GetScreenHeight() * UIParent:GetEffectiveScale()), 
		floor(GetScreenHeight() * UIParent:GetEffectiveScale()),
		1, changeCallback)
	slider:SetRelativeWidth(0.5)
	igroup2:AddChild(slider)
	
	button = UIF:Button(LB.RESET, function() 
		plugin.db:ResetProfile() 
		changeCallback()
	end)
	button:SetUserData("confirm", true)
	button:SetUserData("TriggerGlobalUpdate", true)
	group:AddChild(button)
	
	button = UIF:Button(LB.SHOW, function()
		plugin:SendMessage("RW_StartBiteBuddy")
	end)
	group:AddChild(button)
	
	button = UIF:Button(LB.HIDE, function()
		plugin:SendMessage("RW_StopBiteBuddy")
	end)
	group:AddChild(button)
	
	return group
end