--[[
	News info plguin
]]--


--------------------------------------
-- Create plugin

local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local plugin = RW:Plugin("News")
if not plugin then return end

--------------------------------------
-- Locals
local L = LibStub("AceLocale-3.0"):GetLocale("RW2-Plugin-Base")
local UIF
local AceGUI
local news = {}

--------------------------------------
-- Init

function plugin:OnRegister()
	self.db = RW.db:RegisterNamespace("News", {profile={}})
end

function plugin:PluginEnable()
	UIF = LibStub("AceGUI-Factory-1.0"):GetFactory("RW")
	AceGUI = LibStub("AceGUI-3.0")
end

function plugin:PluginDisable()
end

--------------------------------------
-- Options
function plugin:GetOptions()
	return 98, L.NEWS, self.GetGUI
end

function plugin:GetGUI()
	self = self or plugin
	
	local group, label, color, check
	
	group = AceGUI:Create("ScrollFrame")
	group:SetLayout("Flow")
	
	label = UIF:MainTitleLabel(L.NEWS)
	group:AddChild(label)
	
	local igroup = UIF:InlineGroup("")
	group:AddChild(igroup)	
	
	for title, data in pairs(news) do
		local newsGroup = UIF:InlineGroup2(title)
		igroup:AddChild(newsGroup)
		
		self:GetNewsAsGUI(newsGroup, data, "")
	end
	
	return group
end

function plugin:GetNewsAsGUI(group, data, level)
	for i, subData in pairs(data) do
		if type(subData) == "table" then
			group:AddChild(UIF:NormalText(level.."* "..i))
			plugin:GetNewsAsGUI(group, subData, level.."       ")
		else
			group:AddChild(UIF:NormalText(level.."- "..subData))
		end
	end
end

news["1.21 Beta"] = {
	["ICC"] = {
		["Lord Marrowgar"] = {
			"Adjusted bonestorm cooldown in heroic.",
		},
		["Lady Deathwhisper"] = {
			"Added cooldown and cast timers for Frostbolt in heroic.",
		},
		["Gunship Battle"] = {
			"Health frames will not show up untill the battle starts.",
		},
		["Deathbringer Saurfang"] = {
			"Added cooldown for blood nova (20s)",
			"Adjustd enrage timer in heroic, 6min.",
		},
		["Rotface"] = {
			"Added status bars (health frames) for the Unstable Ooze stacks for each ooze.",
			"Adjusted cooldown on vile gas.",
			"Added personal warning and message about vile gas targets.",
		},
		["Festergut"] = {
			"Added warning for Malleable Goo (heroic)",
		},
		["Professor Putricide"] = {
			"Added personal warnings and direction arrows for the different Variable spells. The direction arrows will point towards the oposit spawnpoint.",
			"Added warnings and raid marks for Unbound Plague.",
		},
		["Blood Prince Council"] = {
			"Added warning and duration timer for Shadow Prison when it reaches 10 stacks or higher.",
			"Corrected spell ID for Empowered Shock Vortex in 10man heroic.",
			"Fixed the cooldown for summoning of Nucleus, off by default.",
		},
		["Sindragosa"] = {
			"Added a timer that shows the duration of Frost Beacon, as when the beacon fades this is when the frost tomb hits.",
			"Added a timer that shows when the next Frost Bomb hits the ground.",
			"Adjusted Blistering Cold cooldown in < 35% phase.",
			"The personal timer for the duration of Buffet should only show up once and not one time per stack.",
		},
		["The Lich King"] = {
			"Added duration timer for Necrotic Plague on players.",
			"Added duration timer for Soul Reaper on tanks.",
			"Added 15min berserk timer.",
			"Added warnings for Shadow Trap (HM), CD, Raid Mark, personal warning, message and Say message when you are the target.",
		},
	},
	["VoA"] = {
		["Toravon"] = {
			"Fixed the spame about Whiteout, should now only be a personal message instead of one message for every raid member."
		},
	},
	["Plugins"] = {
		["Timers"] = {
			"Timers now have an option to flash (change colors) when they reach a specified time left. By default they will start flashing red when they have 10s or less left. The configuration for this is in Plugins -> Alert Plugins -> Timers and find \"Flash\" group in there."
		},
	},
	["General"] = {
		"Added translations for all languages in all instances. (Prepared, so alot still missing)",
		"The message after you wipe will now show the last known boss health percent as well.",
		"Fixed heroic detection.",
		"Enrage timers now shows the time in the options.",
		"Options no longer show each boss 2 times in the dropdown.",
	},
}
news["1.20 Release"] = {
	"Change in folder structure, please delete all old folders that starts with \"RW_\", and also to keep saved variables you need to go into WTF \\Account \\[AccountName] \\SavedVariables \\ and rename RW_Core.lua -> RaidWatch.lua.",
	["ICC"] = {
		["The Lich King"] = {
			"Added warnings and marking for secondary targets of Necrotic Plague.",
			"Added marking on Val'kyrs (Mainly for 25man with 3 of them)",
			"Added marking of the person who will spawn a Raging Spirit.",
			"Added a message with the target of Defile so its easy to see who did not run properly!",
			"It should no longer report it as a wipe when everyone dies to Fury of Frostmourne.",
		},
		["Sindragosa"] = {
			"Added warning, cooldown and personal duration of Mystic Buffet. I will warn at 4 stacks.",
			"Change the sound of the warning when you get pulled in to the \"Come Closer\" warning, not optimal but at least a different sound from the Ice Tomb target warning sound.",
			"Raid marks for Frost Beacon should now be removed when the Beacon is removed.",
		},
		["Lady Deathwhisper"] = {
			"Add waves cooldown keep running in p2 in hard mode, and the CD is only 45s instead of 60.",
			"Added warning when Frostbolt is cast in hard modes.",
		},
		["Lord Marrowgar"] = {
			"Adjusted duration for bonestorm in hard mode, may be totally wrong.",
		},
		["Blood Prince Council"] = {
			"Fixed lua error when standing close to someone who is the target of Shock Vortex, the direction arrow should now pop up and point away from the person."
		},
	},
	["Other"] = {
		"New tab in the options \"News\". Will show the latest changes. Also if you do not have the latest version but someone in your guild or party have a higher version you will see what changes are in those version.",
	}
}
